#tag Module
Protected Module MISC_INTERNAL
	#tag Method, Flags = &h0
		Sub ConfigLoad()
		  
		  Var INI_obj As INI
		  
		  ' Create INI object.
		  '
		  INI_obj = New INI(INIFile_gfi)
		  
		  ' Load configuration.
		  '
		  INI_obj.Load
		  
		  ' Copy INI to global vars.
		  '
		  ConfigMusicPath_gstr = INI_obj.Get("Media", "MusicPath",  "")
		  ConfigMoviePath_gstr = INI_obj.Get("Media", "MoviePath",  "")
		  ConfigDBaseFile_gstr = INI_obj.Get("DBase", "DBaseFile",  "")
		  
		  ' Destroy INI object.
		  '
		  INI_obj = Nil
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Sub CreateMovieTable(SQLiteDatabase_obj As SQLiteDatabase)
		  
		  Var SQL_str As String
		  
		  
		  ' Create movie files table.
		  '
		  SQL_str = "CREATE TABLE `MovieFiles` " _
		  + "(" _
		  + "  `ID`           INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT UNIQUE, " _
		  + "  `File`         TEXT, " _
		  + "  `LastModified` TEXT, " _
		  + "  `MovieArt`     BLOB, " _
		  + "  `MovieName`    TEXT COLLATE NOCASE, " _
		  + "  `MovieYear`    INTEGER, " _
		  + "  `MovieGenre`   TEXT COLLATE NOCASE, " _
		  + "  `MovieTagline` TEXT COLLATE NOCASE" _
		  + ");"
		  
		  Try
		    SQLiteDatabase_obj.ExecuteSQL(SQL_str)
		    
		  Catch error As IOException
		    MessageBox("Create MovieFiles Table error: " + error.Message)
		    Quit
		    
		  End Try
		  
		  ' Create indexes.
		  '
		  SQL_str = "CREATE INDEX `idxMovieName`    ON `MovieFiles` (`MovieName`    COLLATE NOCASE ASC);" _
		  +       "CREATE INDEX `idxMovieYear`    ON `MovieFiles` (`MovieYear`    COLLATE NOCASE ASC);" _
		  +       "CREATE INDEX `idxMovieGenre`   ON `MovieFiles` (`MovieGenre`   COLLATE NOCASE ASC);" _
		  +       "CREATE INDEX `idxMovieTagline` ON `MovieFiles` (`MovieTagline` COLLATE NOCASE ASC);"
		  
		  Try
		    SQLiteDatabase_obj.ExecuteSQL(SQL_str)
		    
		  Catch error As IOException
		    MessageBox("Create MovieFiles Indexes error: " + error.Message)
		    Quit
		    
		  End Try
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Sub CreateMusicTable(SQLiteDatabase_obj As SQLiteDatabase)
		  
		  Var SQL_str As String
		  
		  
		  ' Create music files table.
		  '
		  SQL_str = "CREATE TABLE `MusicFiles` " _
		  + "(" _
		  + "  `ID`           INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT UNIQUE, " _
		  + "  `File`         TEXT, "                _
		  + "  `LastModified` TEXT, "                _
		  + "  `AlbumArt`     BLOB, "                _
		  + "  `AlbumName`    TEXT COLLATE NOCASE, " _
		  + "  `AlbumArtist`  TEXT COLLATE NOCASE, " _
		  + "  `AlbumYear`    INTEGER, "             _
		  + "  `AlbumGenre`   TEXT COLLATE NOCASE, " _
		  + "  `DiscNumber`   INTEGER, "             _
		  + "  `TrackNumber`  INTEGER, "             _
		  + "  `TrackName`    TEXT COLLATE NOCASE, " _
		  + "  `TrackArtist`  TEXT COLLATE NOCASE"   _
		  + ");"
		  
		  Try
		    SQLiteDatabase_obj.ExecuteSQL(SQL_str)
		    
		  Catch error As IOException
		    MessageBox("Create MusicFiles Table error: " + error.Message)
		    Quit
		    
		  End Try
		  
		  ' Create indexes.
		  '
		  SQL_str = "CREATE INDEX `idxAlbumName`   ON `MusicFiles` (`AlbumName`   COLLATE NOCASE ASC);" _
		  +       "CREATE INDEX `idxAlbumArtist` ON `MusicFiles` (`AlbumArtist` COLLATE NOCASE ASC);" _
		  +       "CREATE INDEX `idxAlbumYear`   ON `MusicFiles` (`AlbumYear`   COLLATE NOCASE ASC);" _
		  +       "CREATE INDEX `idxAlbumGenre`  ON `MusicFiles` (`AlbumGenre`  COLLATE NOCASE ASC);" _
		  +       "CREATE INDEX `idxDiscNumber`  ON `MusicFiles` (`DiscNumber`  COLLATE NOCASE ASC);" _
		  +       "CREATE INDEX `idxTrackNumber` ON `MusicFiles` (`TrackNumber` COLLATE NOCASE ASC);" _
		  +       "CREATE INDEX `idxTrackName`   ON `MusicFiles` (`TrackName`   COLLATE NOCASE ASC);" _
		  +       "CREATE INDEX `idxTrackArtist` ON `MusicFiles` (`TrackArtist` COLLATE NOCASE ASC);"
		  
		  Try
		    SQLiteDatabase_obj.ExecuteSQL(SQL_str)
		    
		  Catch error As IOException
		    MessageBox("Create MusicFiles Indexes error: " + error.Message)
		    Quit
		    
		  End Try
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Sub DropMovieTable(SQLiteDatabase_obj As SQLiteDatabase)
		  
		  Var SQL_str As String
		  
		  
		  ' Delete existing movie files table.
		  '
		  SQL_str = "DROP TABLE IF EXISTS `MovieFiles`;"
		  
		  Try
		    SQLiteDatabase_obj.ExecuteSQL(SQL_str)
		    
		  Catch error As IOException
		    MessageBox("Drop MovieFiles Table error: " + error.Message)
		    Quit
		    
		  End Try
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Sub DropMusicTable(SQLiteDatabase_obj As SQLiteDatabase)
		  
		  Var SQL_str As String
		  
		  
		  ' Delete existing music files table.
		  '
		  SQL_str = "DROP TABLE IF EXISTS `MusicFiles`;"
		  
		  Try
		    SQLiteDatabase_obj.ExecuteSQL(SQL_str)
		    
		  Catch error As IOException
		    MessageBox("Drop MusicFiles Table error: " + error.Message)
		    Quit
		    
		  End Try
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Function OpenDatabase(DBaseFile_str As String) As SQLiteDatabase
		  
		  Var SQLiteDatabase_obj As SQLiteDatabase
		  Var FolderItem_obj As FolderItem
		  
		  
		  ' Get folder item for the database file.
		  '
		  FolderItem_obj = New FolderItem(DBaseFile_str, FolderItem.PathModes.Native)
		  'wes-FolderItem_obj = GetFolderItem(DBaseFile_str)
		  
		  ' Create database or connect if already exists.
		  '
		  SQLiteDatabase_obj = New SQLiteDatabase
		  SQLiteDatabase_obj.DatabaseFile = FolderItem_obj
		  
		  Try
		    SQLiteDatabase_obj.CreateDatabase
		    
		  Catch error As IOException
		    MessageBox("Create Database File error: " + error.Message)
		    Quit
		    
		  End Try
		  
		  Return SQLiteDatabase_obj
		  
		End Function
	#tag EndMethod

	#tag Method, Flags = &h0
		Function SafeParseJSON(JSON_txt As Text) As Auto
		  #Pragma BreakOnExceptions Off
		  
		  Try
		    
		    Return ParseJSON(JSON_txt)
		    
		  Catch Error_obj As InvalidJSONException
		    
		    Return New Dictionary
		    
		  Catch Error_obj As RunTimeException
		    
		    Raise Error_obj
		    
		  End Try
		  
		  #Pragma BreakOnExceptions On
		  
		End Function
	#tag EndMethod


	#tag Property, Flags = &h0
		Catalog_genu As Catalog
	#tag EndProperty

	#tag Property, Flags = &h0
		ConfigDBaseFile_gstr As String
	#tag EndProperty

	#tag Property, Flags = &h0
		ConfigMoviePath_gstr As String
	#tag EndProperty

	#tag Property, Flags = &h0
		ConfigMusicPath_gstr As String
	#tag EndProperty

	#tag Property, Flags = &h0
		MainWindow_gobj As main_window = Nil
	#tag EndProperty

	#tag Property, Flags = &h0
		Source_gpic As Picture = Nil
	#tag EndProperty


	#tag Enum, Name = Catalog, Type = Integer, Flags = &h0
		MusicFiles
		MovieFiles
	#tag EndEnum


	#tag ViewBehavior
		#tag ViewProperty
			Name="ConfigDBaseFile_gstr"
			Visible=false
			Group="Behavior"
			InitialValue=""
			Type="String"
			EditorType="MultiLineEditor"
		#tag EndViewProperty
		#tag ViewProperty
			Name="ConfigMoviePath_gstr"
			Visible=false
			Group="Behavior"
			InitialValue=""
			Type="String"
			EditorType="MultiLineEditor"
		#tag EndViewProperty
		#tag ViewProperty
			Name="ConfigMusicPath_gstr"
			Visible=false
			Group="Behavior"
			InitialValue=""
			Type="String"
			EditorType="MultiLineEditor"
		#tag EndViewProperty
		#tag ViewProperty
			Name="Index"
			Visible=true
			Group="ID"
			InitialValue="-2147483648"
			Type="Integer"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="Left"
			Visible=true
			Group="Position"
			InitialValue="0"
			Type="Integer"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="Name"
			Visible=true
			Group="ID"
			InitialValue=""
			Type="String"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="Super"
			Visible=true
			Group="ID"
			InitialValue=""
			Type="String"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="Top"
			Visible=true
			Group="Position"
			InitialValue="0"
			Type="Integer"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="Source_gpic"
			Visible=false
			Group="Behavior"
			InitialValue="Nil"
			Type="Picture"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="Catalog_genu"
			Visible=false
			Group="Behavior"
			InitialValue=""
			Type="Catalog"
			EditorType="Enum"
			#tag EnumValues
				"0 - MusicFiles"
				"1 - MovieFiles"
			#tag EndEnumValues
		#tag EndViewProperty
	#tag EndViewBehavior
End Module
#tag EndModule
