#tag Class
Protected Class amplifier_mgr
	#tag Method, Flags = &h0
		Sub Constructor()
		  
		  ' The BOSE speaker amplifiers are either ON or OFF based on GPIO input. The 
		  ' default state is OFF.
		  '
		  ' To turn the amplifiers ON, the GPIO pin driving the amplifier control relay 
		  ' must be ON.
		  '
		  ' To turn the amplifiers OFF, the GPIO pin driving the amplifier control relay
		  ' must be OFF.
		  
		  
		  ' Set the BOSE speakers amplifier relay control GPIO pin to act as an output.
		  '
		  GPIO.PinMode(ConfigPinAmplifier_gint, GPIO.OUTPUT)
		  
		  ' Turn the BOSE speakers amplifier relay OFF.
		  '
		  GPIO.DigitalWrite(ConfigPinAmplifier_gint, GPIO.OFF)
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Sub Destructor()
		  
		  ' Turn the BOSE speakers amplifier relay OFF.
		  '
		  GPIO.DigitalWrite(ConfigPinAmplifier_gint, GPIO.OFF)
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Sub OFF()
		  
		  ' Set a timer to turn the BOSE speakers amplifier relay OFF after a one second
		  ' delay.
		  '
		  ' If the OFF method is called and then the ON method is called within one
		  ' second, we cancel the timer and the TimerCallLaterAction method is skipped.
		  '
		  ' This is to prevent an unnecessary OFF-to-ON cycle when switching among the 
		  ' three processes that use the BOSE amplified speakers (FM Radio, Music Player,
		  ' and Movie Player.)
		  '
		  ' If the OFF method is called and one second elapses without the ON method 
		  ' being called, then the TimerCallLaterAction method is executed.
		  
		  ' Set a timer to turn the BOSE speaker amplifier relay OFF.
		  '
		  Timer.CallLater(1000, AddressOf Self.TimerCallLaterAction)
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Sub ON()
		  
		  ' Cancel any timer which may have been set by a recent call to the OFF method.
		  '
		  Timer.CancelCallLater(AddressOf Self.TimerCallLaterAction)
		  
		  ' Turn the BOSE speakers amplifier relay ON.
		  '
		  GPIO.DigitalWrite(ConfigPinAmplifier_gint, GPIO.ON)
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h21
		Private Sub TimerCallLaterAction()
		  
		  ' Turn the BOSE speakers amplifier relay OFF.
		  '
		  GPIO.DigitalWrite(ConfigPinAmplifier_gint, GPIO.OFF)
		  
		End Sub
	#tag EndMethod


	#tag ViewBehavior
		#tag ViewProperty
			Name="Index"
			Visible=true
			Group="ID"
			InitialValue="-2147483648"
			Type="Integer"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="Left"
			Visible=true
			Group="Position"
			InitialValue="0"
			Type="Integer"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="Name"
			Visible=true
			Group="ID"
			InitialValue=""
			Type="String"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="Super"
			Visible=true
			Group="ID"
			InitialValue=""
			Type="String"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="Top"
			Visible=true
			Group="Position"
			InitialValue="0"
			Type="Integer"
			EditorType=""
		#tag EndViewProperty
	#tag EndViewBehavior
End Class
#tag EndClass
