#tag Class
Protected Class movie_player_mgr
	#tag Method, Flags = &h21
		Private Sub ConfigurePlayer()
		  
		  Var Lft_int, Top_int, Rgt_int, Bot_int As Integer
		  
		  
		  ' Configure player only if it is currently active (playing or paused).
		  '
		  If (State_penu = States.Playing) Or (State_penu = States.Paused) Then
		    
		    ' Set aspect mode. (options: letterbox, fill, or stretch)
		    '
		    Call OMXCommand("setaspectmode" + " " + ConfigMovieAspectMode_gstr)
		    
		    ' Is video supposed to be shown?
		    '
		    If VideoIsVisible_pboo = True Then
		      
		      ' Size the Movie and place it in the display area.
		      '
		      Select Case FullScreen_pboo
		        
		      Case False 
		        
		        ' Make movie visible inside movie display area.
		        '
		        Var WindowBounds_rct As Rect = MainWindow_gobj.Bounds
		        Lft_int = WindowBounds_rct.Left +  35
		        Top_int = WindowBounds_rct.Top  + 115
		        Rgt_int = WindowBounds_rct.Left + 515
		        Bot_int = WindowBounds_rct.Top  + 358
		        Call OMXCommand("setvideopos " + Lft_int.ToString + " " + Top_int.ToString + " " + Rgt_int.ToString + " " + Bot_int.ToString)
		        
		      Case True
		        
		        ' Make movie visible full-screen.
		        '
		        Var WindowBounds_rct As Rect = MainWindow_gobj.Bounds
		        Lft_int = WindowBounds_rct.Left +   0
		        Top_int = WindowBounds_rct.Top  +   0
		        Rgt_int = WindowBounds_rct.Left + 800
		        Bot_int = WindowBounds_rct.Top  + 480
		        Call OMXCommand("setvideopos " + Lft_int.ToString + " " + Top_int.ToString + " " + Rgt_int.ToString + " " + Bot_int.ToString)
		        
		      End Select
		      
		    Else
		      
		      ' Make movie invisible by placing it off-screen.
		      '
		      Var WindowBounds_rct As Rect = MainWindow_gobj.Bounds
		      Lft_int = WindowBounds_rct.Left + WindowBounds_rct.Width
		      Top_int = WindowBounds_rct.Top  + WindowBounds_rct.Height
		      Rgt_int = WindowBounds_rct.Left + WindowBounds_rct.Width  + 160
		      Bot_int = WindowBounds_rct.Top  + WindowBounds_rct.Height + 100
		      Call OMXCommand("setvideopos " + Lft_int.ToString + " " + Top_int.ToString + " " + Rgt_int.ToString + " " + Bot_int.ToString)
		      
		    End If
		    
		  End If
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Sub Constructor()
		  
		  ' Create a timer to watch the OMXPlayer to detect if it has finished
		  ' playing.
		  '
		  OMXWatchdog_ptmr = New Timer
		  AddHandler OMXWatchdog_ptmr.Action, WeakAddressOf Self.OMXWatchdogAction
		  OMXWatchdog_ptmr.RunMode = Timer.RunModes.Multiple
		  OMXWatchdog_ptmr.Period = 1000
		  OMXWatchdog_ptmr.Enabled = False
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Sub Destructor()
		  
		  ' Shutdown and destroy timer object.
		  '
		  OMXWatchdog_ptmr.Enabled = False
		  RemoveHandler OMXWatchdog_ptmr.Action, WeakAddressOf Self.OMXWatchdogAction
		  OMXWatchdog_ptmr = Nil
		  
		  ' Shutdown OMXPlayer if it is running.
		  '
		  If (State_penu = States.Playing) Or (State_penu = States.Paused) Then
		    State_penu = States.Stopping
		    Call OMXCommand("stop")
		    OMXWaitForRunning(False)
		    State_penu = States.Stopped
		  End If
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Function Duration() As UInt64
		  
		  Return Duration_pint
		  
		End Function
	#tag EndMethod

	#tag Method, Flags = &h0
		Function FullScreen() As Boolean
		  
		  Return FullScreen_pboo
		  
		End Function
	#tag EndMethod

	#tag Method, Flags = &h0
		Sub FullScreen(Assigns NewValue_boo As Boolean)
		  
		  FullScreen_pboo = NewValue_boo
		  
		  ConfigurePlayer
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Function LastFilePlayed() As String
		  
		  Return LastFilePlayed_pstr
		  
		End Function
	#tag EndMethod

	#tag Method, Flags = &h21
		Private Function OMXCommand(Command_str As String) As String
		  
		  Var Shell_obj As Shell
		  Var RetVal_str As String
		  
		  
		  ' Creates a temporary shell so we can invoke the omx_player_dbus_control.sh script and send DBUS commands
		  ' to the OMXPlayer.
		  
		  Shell_obj = New Shell
		  
		  Shell_obj.Execute(AppPath_gstr + "omx_player_dbus_control.sh" + " " + DblQuote("comvette.movie.player") + " " + Command_str)
		  RetVal_str = Shell_obj.Result
		  
		  Shell_obj = Nil
		  
		  Return RetVal_str
		  
		End Function
	#tag EndMethod

	#tag Method, Flags = &h21
		Private Sub OMXWaitForRunning(Value_boo As Boolean)
		  
		  Var RetVal_str As String
		  Var StartTicks_int As Integer
		  Var WaitTicks_int As Integer
		  
		  
		  ' Wait for 20 seconds.
		  '
		  WaitTicks_int = 20 * 60
		  
		  ' Get tick count at start.
		  '
		  StartTicks_int = System.Ticks
		  
		  ' Begin forever loop.
		  '
		  Do
		    
		    ' Poll the OMXPlayer for the media position. If it returns a numeric string we
		    ' know the player is fully instantiated and can be controlled.
		    '
		    RetVal_str = OMXCommand("position").TrimRight
		    If IsNumeric(RetVal_str) = Value_boo Then
		      Exit Do
		    End If
		    
		    ' Have we tried for more than 20 seconds to get a response?
		    '
		    If System.Ticks > (StartTicks_int + WaitTicks_int) Then
		      Var AckBox_obj As ack_box = New ack_box
		      AckBox_obj.ShowModal("Timeout Error", "OMXPlayer failed to respond within 20 seconds." + EndOfLine + "Application will close.", ack_box.MsgTypes.Emergency)
		      Quit
		    End If
		    
		  Loop
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h21
		Private Sub OMXWatchdogAction(Instance_tmr As Timer)
		  
		  #Pragma Unused Instance_tmr
		  
		  
		  ' Poll the OMXPlayer for the media position (when it stops returning a numeric string 
		  ' we know the player has finished playing) and then raise the MediaCompleted event to 
		  ' send a notification to the host of this class.
		  
		  Var RetVal_str As String = OMXCommand("position").TrimRight
		  If IsNumeric(RetVal_str) = False Then
		    OMXWatchdog_ptmr.Enabled = False
		    State_penu = States.Stopped
		    RaiseEvent MediaCompleted
		    Exit Sub
		  End If
		  
		  ' Raise media progress event to our host.
		  '
		  Var Position_int As UInt64 = CType(Val(RetVal_str), UInt64)
		  Var Percentage_int As Integer = Position_int / (Duration_pint * .01)
		  RaiseEvent MediaProgress(Percentage_int)
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Sub Pause()
		  
		  ' Pause playing.
		  '
		  If State_penu = States.Playing Then
		    State_penu = States.Pausing
		    OMXWatchdog_ptmr.Enabled = False
		    Call OMXCommand("pause")
		    State_penu = States.Paused
		  End If
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Sub Play(File_str As String = "")
		  
		  ' Play method logic:
		  '
		  ' If the Movie player state is "stopped", then play the provided file. If no file was provided, 
		  ' then exit the Play method.
		  '
		  ' If the Movie player state is "paused", then resume playing the current file.
		  '
		  ' If the Movie player state is "playing", then stop playing the current file. If a file was
		  ' provided, then play the provided file, otherwise restart the current file.
		  '
		  Select Case State_penu
		    
		  Case States.Stopped
		    
		    ' Start playing new media file if provided.
		    '
		    If File_str <> "" Then
		      
		      State_penu = States.Starting
		      WriteMovieLauncherData(File_str)
		      Duration_pint = Val(OMXCommand("duration"))
		      OMXWatchdog_ptmr.Enabled = True
		      State_penu = States.Playing
		      
		      LastFilePlayed_pstr = File_str
		      
		      ConfigurePlayer
		      
		    End If
		    
		  Case States.Paused
		    
		    ' Resume playing current media file.
		    '
		    State_penu = States.Resuming
		    Call OMXCommand("play")
		    OMXWatchdog_ptmr.Enabled = True
		    State_penu = States.Playing
		    
		  Case States.Playing
		    
		    ' Stop playing current media file.
		    '
		    State_penu = States.Stopping
		    OMXWatchdog_ptmr.Enabled = False
		    Call OMXCommand("stop")
		    OMXWaitForRunning(False)
		    State_penu = States.Stopped
		    
		    ' Start playing new media file if provided. Otherwise restart
		    ' current media file.
		    '
		    If File_str <> "" Then
		      
		      State_penu = States.Starting
		      WriteMovieLauncherData(File_str)
		      Duration_pint = Val(OMXCommand("duration"))
		      OMXWatchdog_ptmr.Enabled = True
		      State_penu = States.Playing
		      
		      LastFilePlayed_pstr = File_str
		      
		      ConfigurePlayer
		      
		    Else
		      
		      State_penu = States.Starting
		      WriteMovieLauncherData(LastFilePlayed_pstr)
		      OMXWatchdog_ptmr.Enabled = True
		      State_penu = States.Playing
		      
		      ConfigurePlayer
		      
		    End If
		    
		    ConfigurePlayer
		    
		  End Select
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Sub SetPosition(MicroSeconds_int As UInt64)
		  
		  ' Is specified position withing acceptable range?
		  '
		  If (MicroSeconds_int < 0) Or (MicroSeconds_int > Duration_pint) Then
		    Exit Sub
		  End If
		  
		  ' Set new Movie position.
		  '
		  If State_penu = States.Playing Then
		    State_penu = States.Positioning
		    OMXWatchdog_ptmr.Enabled = False
		    Call OMXCommand("setposition" + " " + Str(MicroSeconds_int))
		    OMXWaitForRunning(True)
		    OMXWatchdog_ptmr.Enabled = True
		    State_penu = States.Playing
		  End If
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Function State() As movie_player_mgr.States
		  
		  Return State_penu
		  
		End Function
	#tag EndMethod

	#tag Method, Flags = &h0
		Sub Stop()
		  
		  ' Stop playing.
		  '
		  If (State_penu = States.Playing) Or (State_penu = States.Paused) Then
		    State_penu = States.Stopping
		    OMXWatchdog_ptmr.Enabled = False
		    Call OMXCommand("stop")
		    State_penu = States.Stopped
		  End If
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Function VideoIsVisible() As Boolean
		  
		  Return VideoIsVisible_pboo
		  
		End Function
	#tag EndMethod

	#tag Method, Flags = &h0
		Sub VideoIsVisible(Assigns NewValue_boo As Boolean)
		  
		  Var Lft_int, Top_int, Rgt_int, Bot_int As Integer
		  
		  
		  ' Set new visibility.
		  '
		  VideoIsVisible_pboo = NewValue_boo
		  
		  ' Configure player only if it is currently active (playing or paused).
		  '
		  If (State_penu = States.Playing) Or (State_penu = States.Paused) Then
		    
		    Select Case VideoIsVisible_pboo
		      
		    Case True
		      
		      ' Make movie visible by placing it on-screen.
		      '
		      Select Case FullScreen_pboo
		        
		      Case False 
		        
		        ' Make movie visible inside movie display area.
		        '
		        Var WindowBounds_rct As Rect = MainWindow_gobj.Bounds
		        Lft_int = WindowBounds_rct.Left +  35
		        Top_int = WindowBounds_rct.Top  + 115
		        Rgt_int = WindowBounds_rct.Left + 515
		        Bot_int = WindowBounds_rct.Top  + 358
		        Call OMXCommand("setvideopos " + Lft_int.ToString + " " + Top_int.ToString + " " + Rgt_int.ToString + " " + Bot_int.ToString)
		        
		      Case True
		        
		        ' Make movie visible full-screen.
		        '
		        Var WindowBounds_rct As Rect = MainWindow_gobj.Bounds
		        Lft_int = WindowBounds_rct.Left +   0
		        Top_int = WindowBounds_rct.Top  +   0
		        Rgt_int = WindowBounds_rct.Left + 800
		        Bot_int = WindowBounds_rct.Top  + 480
		        Call OMXCommand("setvideopos " + Lft_int.ToString + " " + Top_int.ToString + " " + Rgt_int.ToString + " " + Bot_int.ToString)
		        
		      End Select
		      
		    Case False
		      
		      ' Make movie invisible by placing it off-screen.
		      '
		      Var WindowBounds_rct As Rect = MainWindow_gobj.Bounds
		      Lft_int = WindowBounds_rct.Left + WindowBounds_rct.Width
		      Top_int = WindowBounds_rct.Top  + WindowBounds_rct.Height
		      Rgt_int = WindowBounds_rct.Left + WindowBounds_rct.Width  + 160
		      Bot_int = WindowBounds_rct.Top  + WindowBounds_rct.Height + 100
		      Call OMXCommand("setvideopos " + Lft_int.ToString + " " + Top_int.ToString + " " + Rgt_int.ToString + " " + Bot_int.ToString)
		      
		    End Select
		    
		  End If
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h21
		Private Sub WriteMovieLauncherData(Data_str As String)
		  
		  ' Creates a launch_movie.dat file and writes the name of the movie file to be played
		  ' into it. 
		  '
		  ' The launch_movie.sh file is always running as a background process waiting for a 
		  ' launch_movie.dat file to appear. When it finds a launch_movie.dat file it will
		  ' launch the OMXPlayer with the Movie file named in the launch_movie.dat file.
		  '
		  ' Once the launch_movie.sh process launches the OMXPlayer, it pauses until the 
		  ' OMXPlayer is finished playing the movie file at which time it will return to
		  ' waiting for a new launch_movie.dat file to appear.
		  
		  
		  ' Create a launch_movie.dat file.
		  '
		  StringToFile(CurrentMethodName, AppPath_gstr + "launch_movie.dat", Data_str)
		  
		  OMXWaitForRunning(True)
		  
		  ' Once the OMXPlayer is running, delete the launch_movie.dat file. It has served it's
		  ' purpose and is no longer needed.
		  '
		  DeleteFile(AppPath_gstr + "launch_movie.dat")
		  
		End Sub
	#tag EndMethod


	#tag Hook, Flags = &h0
		Event MediaCompleted()
	#tag EndHook

	#tag Hook, Flags = &h0
		Event MediaProgress(NewPercentage_int As Integer)
	#tag EndHook


	#tag Property, Flags = &h21
		Private Duration_pint As UInt64 = 0
	#tag EndProperty

	#tag Property, Flags = &h21
		Private FullScreen_pboo As Boolean = False
	#tag EndProperty

	#tag Property, Flags = &h21
		Private LastFilePlayed_pstr As String
	#tag EndProperty

	#tag Property, Flags = &h21
		Private OMXWatchdog_ptmr As Timer = Nil
	#tag EndProperty

	#tag Property, Flags = &h21
		Private State_penu As States = States.Stopped
	#tag EndProperty

	#tag Property, Flags = &h21
		Private VideoIsVisible_pboo As boolean = False
	#tag EndProperty


	#tag Enum, Name = States, Type = Integer, Flags = &h0
		Stopping
		  Stopped
		  Pausing
		  Paused
		  Resuming
		  Starting
		  Playing
		Positioning
	#tag EndEnum


	#tag ViewBehavior
		#tag ViewProperty
			Name="Index"
			Visible=true
			Group="ID"
			InitialValue="-2147483648"
			Type="Integer"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="Left"
			Visible=true
			Group="Position"
			InitialValue="0"
			Type="Integer"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="Name"
			Visible=true
			Group="ID"
			InitialValue=""
			Type="String"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="Super"
			Visible=true
			Group="ID"
			InitialValue=""
			Type="String"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="Top"
			Visible=true
			Group="Position"
			InitialValue="0"
			Type="Integer"
			EditorType=""
		#tag EndViewProperty
	#tag EndViewBehavior
End Class
#tag EndClass
