#tag Class
Protected Class music_player_mgr
	#tag Method, Flags = &h0
		Sub Constructor()
		  
		  ' Create a timer to watch the OMXPlayer to detect if it has finished
		  ' playing.
		  '
		  OMXWatchdog_ptmr = New Timer
		  AddHandler OMXWatchdog_ptmr.Action, WeakAddressOf Self.OMXWatchdogAction
		  OMXWatchdog_ptmr.RunMode = Timer.RunModes.Multiple
		  OMXWatchdog_ptmr.Period = 1000
		  OMXWatchdog_ptmr.Enabled = False
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Sub Destructor()
		  
		  ' Shutdown and destroy timer object.
		  '
		  OMXWatchdog_ptmr.Enabled = False
		  RemoveHandler OMXWatchdog_ptmr.Action, WeakAddressOf Self.OMXWatchdogAction
		  OMXWatchdog_ptmr = Nil
		  
		  ' Shutdown OMXPlayer if it is running.
		  '
		  If (State_penu = States.Playing) Or (State_penu = States.Paused) Then
		    State_penu = States.Stopping
		    Call OMXCommand("stop")
		    OMXWaitForRunning(False)
		    State_penu = States.Stopped
		  End If
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Function Duration() As UInt64
		  
		  Return Duration_pint
		  
		End Function
	#tag EndMethod

	#tag Method, Flags = &h0
		Function LastFilePlayed() As String
		  
		  Return LastFilePlayed_pstr
		  
		End Function
	#tag EndMethod

	#tag Method, Flags = &h21
		Private Function OMXCommand(Command_str As String) As String
		  
		  Var Shell_obj As Shell
		  Var RetVal_str As String
		  
		  
		  ' Creates a temporary shell so we can invoke the omx_player_dbus_control.sh script and send DBUS commands
		  ' to the OMXPlayer.
		  
		  Shell_obj = New Shell
		  
		  Shell_obj.Execute(AppPath_gstr + "omx_player_dbus_control.sh" + " " + DblQuote("comvette.music.player") + " " + Command_str)
		  RetVal_str = Shell_obj.Result
		  
		  Shell_obj = Nil
		  
		  Return RetVal_str
		  
		End Function
	#tag EndMethod

	#tag Method, Flags = &h21
		Private Sub OMXWaitForRunning(Value_boo As Boolean)
		  
		  Var RetVal_str As String
		  Var StartTicks_int As Integer
		  Var WaitTicks_int As Integer
		  
		  
		  ' Wait for 20 seconds.
		  '
		  WaitTicks_int = 20 * 60
		  
		  ' Get tick count at start.
		  '
		  StartTicks_int = System.Ticks
		  
		  ' Begin forever loop.
		  '
		  Do
		    
		    ' Poll the OMXPlayer for the media position. If it returns a numeric string we
		    ' know the player is fully instantiated and can be controlled.
		    '
		    RetVal_str = OMXCommand("position").TrimRight
		    If IsNumeric(RetVal_str) = Value_boo Then
		      Exit Do
		    End If
		    
		    ' Have we tried for more than 20 seconds to get a response?
		    '
		    If System.Ticks > (StartTicks_int + WaitTicks_int) Then
		      Var AckBox_obj As ack_box = New ack_box
		      AckBox_obj.ShowModal("Timeout Error", "OMXPlayer failed to respond within 20 seconds." + EndOfLine + "Application will close.", ack_box.MsgTypes.Emergency)
		      Quit
		    End If
		    
		  Loop
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h21
		Private Sub OMXWatchdogAction(Instance_tmr As Timer)
		  
		  #Pragma Unused Instance_tmr
		  
		  
		  ' Poll the OMXPlayer for the media position (when it stops returning a numeric string 
		  ' we know the player has finished playing) and then raise the MediaCompleted event to 
		  ' send a notification to the host of this class.
		  
		  Var RetVal_str As String = OMXCommand("position").TrimRight
		  If IsNumeric(RetVal_str) = False Then
		    OMXWatchdog_ptmr.Enabled = False
		    State_penu = States.Stopped
		    RaiseEvent MediaCompleted
		    Exit Sub
		  End If
		  
		  ' Raise media progress event to our host.
		  '
		  Var Position_int As UInt64 = CType(Val(RetVal_str), UInt64)
		  Var Percentage_int As Integer = Position_int / (Duration_pint * .01)
		  RaiseEvent MediaProgress(Percentage_int)
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Sub Pause()
		  
		  ' Pause playing.
		  '
		  If State_penu = States.Playing Then
		    State_penu = States.Pausing
		    OMXWatchdog_ptmr.Enabled = False
		    Call OMXCommand("pause")
		    State_penu = States.Paused
		  End If
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Sub Play(File_str As String = "")
		  
		  ' Play method logic:
		  '
		  ' If the Music player state is "stopped", then play the provided file. If no file was provided, 
		  ' then exit the Play method.
		  '
		  ' If the Music player state is "paused", then resume playing the current file.
		  '
		  ' If the Music player state is "playing", then stop playing the current file. If a file was
		  ' provided, then play the provided file, otherwise restart the current file.
		  '
		  Select Case State_penu
		    
		  Case States.Stopped
		    
		    ' Start playing new media file if provided.
		    '
		    If File_str <> "" Then
		      
		      State_penu = States.Starting
		      WriteMusicLauncherData(File_str)
		      Duration_pint = Val(OMXCommand("duration"))
		      OMXWatchdog_ptmr.Enabled = True
		      State_penu = States.Playing
		      
		      LastFilePlayed_pstr = File_str
		      
		    End If
		    
		  Case States.Paused
		    
		    ' Resume playing current media file.
		    '
		    State_penu = States.Resuming
		    Call OMXCommand("play")
		    OMXWatchdog_ptmr.Enabled = True
		    State_penu = States.Playing
		    
		  Case States.Playing
		    
		    ' Stop playing current media file.
		    '
		    State_penu = States.Stopping
		    OMXWatchdog_ptmr.Enabled = False
		    Call OMXCommand("stop")
		    OMXWaitForRunning(False)
		    State_penu = States.Stopped
		    
		    ' Start playing new media file if provided. Otherwise restart
		    ' current media file.
		    '
		    If File_str <> "" Then
		      
		      State_penu = States.Starting
		      WriteMusicLauncherData(File_str)
		      Duration_pint = Val(OMXCommand("duration"))
		      OMXWatchdog_ptmr.Enabled = True
		      State_penu = States.Playing
		      
		      LastFilePlayed_pstr = File_str
		      
		    Else
		      
		      State_penu = States.Starting
		      WriteMusicLauncherData(LastFilePlayed_pstr)
		      OMXWatchdog_ptmr.Enabled = True
		      State_penu = States.Playing
		      
		    End If
		    
		  End Select
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Function State() As music_player_mgr.States
		  
		  Return State_penu
		  
		End Function
	#tag EndMethod

	#tag Method, Flags = &h0
		Sub Stop()
		  
		  ' Stop playing.
		  '
		  If (State_penu = States.Playing) Or (State_penu = States.Paused) Then
		    State_penu = States.Stopping
		    OMXWatchdog_ptmr.Enabled = False
		    Call OMXCommand("stop")
		    State_penu = States.Stopped
		  End If
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h21
		Private Sub WriteMusicLauncherData(Data_str As String)
		  
		  ' Creates a launch_music.dat file and writes the name of the music file to be played
		  ' into it. 
		  '
		  ' The launch_music.sh file is always running as a background process waiting for a 
		  ' launch_music.dat file to appear. When it finds a launch_music.dat file it will
		  ' launch the OMXPlayer with the music file named in the launch_music.dat file.
		  '
		  ' Once the launch_music.sh process launches the OMXPlayer, it pauses until the 
		  ' OMXPlayer is finished playing the music file at which time it will return to
		  ' waiting for a new launch_music.dat file to appear.
		  
		  
		  ' Create a launch_music.dat file.
		  '
		  StringToFile(CurrentMethodName, AppPath_gstr + "launch_music.dat", Data_str)
		  
		  OMXWaitForRunning(True)
		  
		  ' Once the OMXPlayer is running, delete the launch_music.dat file. It has served it's
		  ' purpose and is no longer needed.
		  '
		  DeleteFile(AppPath_gstr + "launch_music.dat")
		  
		End Sub
	#tag EndMethod


	#tag Hook, Flags = &h0
		Event MediaCompleted()
	#tag EndHook

	#tag Hook, Flags = &h0
		Event MediaProgress(NewPercentage_int As Integer)
	#tag EndHook


	#tag Property, Flags = &h21
		Private Duration_pint As UInt64 = 0
	#tag EndProperty

	#tag Property, Flags = &h21
		Private LastFilePlayed_pstr As String
	#tag EndProperty

	#tag Property, Flags = &h21
		Private OMXWatchdog_ptmr As Timer = Nil
	#tag EndProperty

	#tag Property, Flags = &h21
		Private State_penu As States = States.Stopped
	#tag EndProperty


	#tag Enum, Name = States, Type = Integer, Flags = &h0
		Stopping
		  Stopped
		  Pausing
		  Paused
		  Resuming
		  Starting
		Playing
	#tag EndEnum


	#tag ViewBehavior
		#tag ViewProperty
			Name="Index"
			Visible=true
			Group="ID"
			InitialValue="-2147483648"
			Type="Integer"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="Left"
			Visible=true
			Group="Position"
			InitialValue="0"
			Type="Integer"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="Name"
			Visible=true
			Group="ID"
			InitialValue=""
			Type="String"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="Super"
			Visible=true
			Group="ID"
			InitialValue=""
			Type="String"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="Top"
			Visible=true
			Group="Position"
			InitialValue="0"
			Type="Integer"
			EditorType=""
		#tag EndViewProperty
	#tag EndViewBehavior
End Class
#tag EndClass
