#tag Class
Protected Class radio_mgr
	#tag Method, Flags = &h0
		Sub Constructor()
		  
		  ' Generate array of valid frequencies.
		  '
		  GenerateValidFrequencies
		  
		  ' Initialize the TEA5767 FM radio code module.
		  '
		  TEA5767.Init
		  
		  ' Initialize the TEA5767 FM radio hardware module.
		  '
		  TEA5767.TXBandLimits            = TEA5767.BandLimits.USA_EUROPE
		  TEA5767.TXCrystal               = TEA5767.Crystal.SLOW
		  TEA5767.TXDeEmphasis            = TEA5767.DeEmphasis.USA
		  TEA5767.TXHighCutControl        = TEA5767.HighCutControl.ON
		  TEA5767.TXMute                  = TEA5767.Mute.OFF
		  TEA5767.TXMuteLeft              = TEA5767.MuteLeft.ON
		  TEA5767.TXMuteRight             = TEA5767.MuteRight.ON
		  TEA5767.TXPin14Mode             = TEA5767.Pin14Mode.GPIOPort
		  TEA5767.TXPLLHi                 = 0
		  TEA5767.TXPLLLo                 = 0
		  TEA5767.TXPLLRef                = TEA5767.PLLRef.Disabled
		  TEA5767.TXPort1                 = TEA5767.Port1.LO
		  TEA5767.TXPort2                 = TEA5767.Port2.LO
		  TEA5767.TXReception             = TEA5767.Reception.STEREO
		  TEA5767.TXSearchDirection       = TEA5767.SearchDirection.UP
		  TEA5767.TXSearchMode            = TEA5767.SearchMode.OFF
		  TEA5767.TXSearchStopLevel       = TEA5767.SearchStopLevel.HIGH
		  TEA5767.TXSideInjection         = TEA5767.SideInjection.LO
		  TEA5767.TXSoftMute              = TEA5767.SoftMute.ON
		  TEA5767.TXStandby               = TEA5767.Standby.OFF
		  TEA5767.TXStereoNoiseCancelling = TEA5767.StereoNoiseCancelling.ON
		  
		  TEA5767.DataTX
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h21
		Private Sub ConvertFrequencyToPLL(Frequency_int As UInt32, ByRef PLLHi_int As UInt8, ByRef PLLLo_int As UInt8)
		  
		  Var PLLHiLo_int As UInt16
		  
		  
		  ' Caller provides a Frequency_int and this routine updates ByRef the values
		  ' in PLLHi_int and PLLLo_int. The caller then uses the PLLLo_int and PLLHi_int
		  ' values.
		  
		  ' Calculate Phase-Lock-Loop value from the specified Frequency.
		  '
		  If TEA5767.TXSideInjection = TEA5767.SideInjection.LO Then
		    
		    PLLHiLo_int = CType(((Frequency_int - INTERMEDIATE_FREQUENCY) * 4) / CRYSTAL_FREQUENCY, UInt16)
		    
		  Else
		    
		    PLLHiLo_int = CType(((Frequency_int + INTERMEDIATE_FREQUENCY) * 4) / CRYSTAL_FREQUENCY, UInt16)
		    
		  End If
		  
		  ' Get High byte of PLLHiLo_int
		  '
		  PLLHi_int = Bitwise.ShiftRight(PLLHiLo_int, 8)
		  
		  ' Get Low byte of PLLHiLo_int
		  '
		  PLLLo_int = PLLHiLo_int
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h21
		Private Sub ConvertPLLToFrequency(PLLHi_int As UInt8, PLLLo_int As UInt8, ByRef Frequency_int As UInt32)
		  
		  Var PLLHiLo_int As UInt16
		  
		  
		  ' Caller provides PLLHi_int and PLLLo_int and this routine updates ByRef the value
		  ' in Frequency_int. The caller then uses the Frequency_int value.
		  
		  ' Combine PLL_Hi and PLL_Lo bytes.
		  '
		  PLLHiLo_int = Bitwise.ShiftLeft(PLLHi_int And &b00111111, 8) + PLLLo_int
		  
		  ' Calculate Frequency from the specified Phase-Lock-Loop value.
		  '
		  If TEA5767.TXSideInjection = TEA5767.SideInjection.LO Then
		    
		    Frequency_int  = (((PLLHiLo_int * CRYSTAL_FREQUENCY) / 4) + INTERMEDIATE_FREQUENCY)
		    
		  Else
		    
		    Frequency_int  = (((PLLHiLo_int * CRYSTAL_FREQUENCY) / 4) - INTERMEDIATE_FREQUENCY)
		    
		  End If
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Sub Destructor()
		  
		  ' Mute the radio.
		  '
		  TEA5767.TXMute = TEA5767.Mute.ON
		  TEA5767.DataTX
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Sub Down()
		  
		  Var PLLHi_int As UInt8
		  Var PLLLo_int As UInt8
		  
		  
		  ' Decrement frequency.
		  '
		  Frequency_pint = FindNextValidFrequencyDown(Frequency_pint)
		  
		  ' Tune the radio.
		  '
		  ConvertFrequencyToPLL(Frequency_pint, PLLHi_int, PLLLo_int)
		  TEA5767.TXPLLHi = PLLHi_int
		  TEA5767.TXPLLLo = PLLLo_int
		  TEA5767.DataTX
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h21
		Private Function FindNextValidFrequencyDown(CurrentFrequency_int As UInt32) As UInt32
		  
		  Var RetVal_int As UInt32
		  
		  
		  ' Scan down the list of valid frequencies and find the first
		  ' frequency that is less than the current frequency. That's
		  ' the next valid frequency, so return it. If no lesser valid
		  ' frequency is found then return the maximum valid frequency.
		  
		  RetVal_int = MAXIMUM_FREQUENCY
		  
		  For Index_int As Integer = ValidFrequencies_pint.LastIndex DownTo ValidFrequencies_pint.FirstRowIndex
		    
		    If ValidFrequencies_pint(Index_int) < CurrentFrequency_int Then
		      RetVal_int = ValidFrequencies_pint(Index_int)
		      Exit For
		    End If
		    
		  Next Index_int
		  
		  Return RetVal_int
		  
		End Function
	#tag EndMethod

	#tag Method, Flags = &h21
		Private Function FindNextValidFrequencyUp(CurrentFrequency_int As UInt32) As UInt32
		  
		  Var RetVal_int As UInt32
		  
		  
		  ' Scan up the list of valid frequencies and find the first
		  ' frequency that is greater than the current frequency. That's
		  ' the next valid frequency, so return it. If no greater valid
		  ' frequency is found then return the minimum valid frequency.
		  
		  RetVal_int = MINIMUM_FREQUENCY
		  
		  For Index_int As Integer = ValidFrequencies_pint.FirstRowIndex To ValidFrequencies_pint.LastIndex
		    
		    If ValidFrequencies_pint(Index_int) > CurrentFrequency_int Then
		      RetVal_int = ValidFrequencies_pint(Index_int)
		      Exit For
		    End If
		    
		  Next Index_int
		  
		  Return RetVal_int
		  
		End Function
	#tag EndMethod

	#tag Method, Flags = &h0
		Function Frequency() As UInt32
		  
		  Return Frequency_pint
		  
		End Function
	#tag EndMethod

	#tag Method, Flags = &h0
		Sub Frequency(Assigns NewValue_int As UInt32)
		  
		  Var PLLHi_int As UInt8
		  Var PLLLo_int As UInt8
		  
		  
		  Frequency_pint = NewValue_int
		  
		  ' Did user enter a valid frequency value?
		  '
		  If Frequency_pint < radio_mgr.MINIMUM_FREQUENCY Then
		    Frequency_pint = radio_mgr.MINIMUM_FREQUENCY
		  End If
		  
		  If Frequency_pint > radio_mgr.MAXIMUM_FREQUENCY Then
		    Frequency_pint = radio_mgr.MAXIMUM_FREQUENCY
		  End If
		  
		  ' Tune the radio.
		  '
		  ConvertFrequencyToPLL(Frequency_pint, PLLHi_int, PLLLo_int)
		  TEA5767.TXPLLHi = PLLHi_int
		  TEA5767.TXPLLLo = PLLLo_int
		  TEA5767.DataTX
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h21
		Private Sub GenerateValidFrequencies()
		  
		  Var Frequency_int As UInt32
		  
		  
		  ' Build a list of valid frequencies starting at the minimum
		  ' frequency, spaced at the step interval. 
		  '
		  Var LoopCount_int As Integer
		  Do
		    
		    Frequency_int = MINIMUM_FREQUENCY + (LoopCount_int * FREQUENCY_STEP)
		    
		    If Frequency_int > MAXIMUM_FREQUENCY Then
		      Exit
		    End If
		    
		    ValidFrequencies_pint.Add(Frequency_int)
		    
		    LoopCount_int = LoopCount_int + 1
		    
		  Loop
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Sub Mute()
		  
		  TEA5767.TXMuteLeft = TEA5767.MuteLeft.ON
		  TEA5767.TXMuteRight = TEA5767.MuteRight.ON
		  TEA5767.DataTX
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Sub Search()
		  
		  Var PLLHi_int As UInt8
		  Var PLLLo_int As UInt8
		  Var NeighborDn_int As UInt32
		  Var NeighborUp_int As UInt32
		  Var EndOfBand_int As UInt8
		  
		  ' Perform an autonomous station search.
		  '
		  Do
		    
		    ' Depending on search direction flag either increment
		    ' or decrement the MHz value by 100000 (.1MHz)
		    '
		    Select Case TEA5767.TXSearchDirection
		      
		    Case TEA5767.SearchDirection.DOWN
		      Frequency_pint = Frequency_pint - 100000
		      
		    Case TEA5767.SearchDirection.UP
		      Frequency_pint = Frequency_pint + 100000
		      
		    End Select
		    
		    ' Generate new PLL_Hi and PLL_Lo bits from frequency.
		    '
		    ConvertFrequencyToPLL(Frequency_pint, PLLHi_int, PLLLo_int)
		    
		    ' Now set the conditions needed for the radio to find the 
		    ' next available station.
		    '
		    TEA5767.TXPLLHi = PLLHi_int
		    TEA5767.TXPLLLo = PLLLo_int
		    TEA5767.TXSearchMode = TEA5767.SearchMode.ON
		    TEA5767.TXMute = TEA5767.Mute.ON
		    TEA5767.DataTX
		    
		    ' Wait while the radio searches for a station.
		    '
		    WaitForRadioReady
		    
		    ' Read the radio.
		    '
		    TEA5767.DataRX
		    
		    ' Did we hit the end of the frequency band?
		    '
		    If TEA5767.RXBandLimitFlag = TEA5767.BandLimitFlag.HI Then
		      
		      ' Keep track of how many times we've hit the end of
		      ' the frequency band.
		      '
		      EndOfBand_int = EndOfBand_int + 1
		      
		      ' If we've hit the end of the frequency band a second
		      ' time then no stations were found.
		      '
		      If EndOfBand_int = 2 Then
		        Exit Do
		      End If
		      
		      ' End of frequency band was hit, so go to other end of 
		      ' frequency band and continue searching.
		      '
		      Select Case TEA5767.TXSearchDirection
		        
		      Case TEA5767.SearchDirection.DOWN
		        Frequency_pint = MAXIMUM_FREQUENCY
		        
		      Case TEA5767.SearchDirection.UP
		        Frequency_pint = MINIMUM_FREQUENCY
		        
		      End Select
		      
		      Continue Do
		      
		    End If
		    
		    Exit Do
		    
		  Loop
		  
		  ' Get PLL bits for the currently tuned frequency.
		  '
		  PLLHi_int = TEA5767.RXPLLHi
		  PLLLo_int = TEA5767.RXPLLLo
		  
		  ' Set the PLL, and un-set the mute and search.
		  '
		  TEA5767.TXPLLHi = PLLHi_int
		  TEA5767.TXPLLLo = PLLLo_int
		  TEA5767.TXSearchMode = TEA5767.SearchMode.OFF
		  TEA5767.TXMute = TEA5767.Mute.OFF
		  TEA5767.DataTX
		  
		  ' What frequency did the tuner land on?
		  '
		  ConvertPLLToFrequency(PLLHi_int, PLLLo_int, Frequency_pint)
		  
		  ' Since a PLL value cannot be converted back into an frequency
		  ' that falls precisely on a valid frequency we need to find its
		  ' nearest neighbor frequency and adjust our frequency value to it.
		  
		  ' Get the nearest valid neighboring frequencies above and below
		  ' the frequency reported by the tuner.
		  '
		  NeighborDn_int = FindNextValidFrequencyDown(Frequency_pint) 
		  NeighborUp_int = FindNextValidFrequencyUp(Frequency_pint) 
		  
		  ' Adjust our frequency to its nearest neighbor.
		  
		  If (Frequency_pint - NeighborDn_int) < (NeighborUp_int - Frequency_pint) Then
		    Frequency_pint = NeighborDn_int
		  Else
		    Frequency_pint = NeighborUp_int
		  End If
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Sub SearchStopLevel(Assigns NewValue_str As String)
		  
		  Select Case NewValue_str
		    
		  Case "low"
		    TEA5767.TXSearchStopLevel = TEA5767.SearchStopLevel.LOW
		    
		  Case "medium"
		    TEA5767.TXSearchStopLevel = TEA5767.SearchStopLevel.MEDIUM
		    
		  Case "high"
		    TEA5767.TXSearchStopLevel = TEA5767.SearchStopLevel.HIGH
		    
		  End Select
		  
		  TEA5767.DataTX
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Sub Status(ByRef IsStereo_boo As Boolean, ByRef SignalLevel_int As UInt8)
		  
		  TEA5767.DataRX
		  
		  Select Case TEA5767.RXStereoFlag
		    
		  Case TEA5767.StereoFlag.LO
		    IsStereo_boo = False
		    
		  Case TEA5767.StereoFlag.HI
		    IsStereo_boo = True
		    
		  End Select
		  
		  SignalLevel_int = TEA5767.RXSignalLevel
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Sub UnMute()
		  
		  TEA5767.TXMuteLeft = TEA5767.MuteLeft.OFF
		  TEA5767.TXMuteRight = TEA5767.MuteRight.OFF
		  TEA5767.DataTX
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Sub Up()
		  
		  Var PLLHi_int As UInt8
		  Var PLLLo_int As UInt8
		  
		  
		  ' Increment frequency.
		  '
		  Frequency_pint = FindNextValidFrequencyUp(Frequency_pint)
		  
		  ' Tune the radio.
		  '
		  ConvertFrequencyToPLL(Frequency_pint, PLLHi_int, PLLLo_int)
		  TEA5767.TXPLLHi = PLLHi_int
		  TEA5767.TXPLLLo = PLLLo_int
		  TEA5767.DataTX
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h21
		Private Sub WaitForRadioReady()
		  
		  Var LoopCount_int As Integer
		  
		  Do
		    
		    ' Pause for 1/10th of a second.
		    '
		    GPIO.Delay(100)
		    
		    ' Exit if 10 seconds has elapsed.
		    '
		    LoopCount_int = LoopCount_int + 1
		    If LoopCount_int > 100 Then
		      Exit Do
		    End If
		    
		    ' Exit if the radio is ready.
		    '
		    TEA5767.DataRX
		    If TEA5767.RXReadyFlag = TEA5767.ReadyFlag.HI Then
		      Exit Do
		    End If
		    
		  Loop
		  
		End Sub
	#tag EndMethod


	#tag Property, Flags = &h21
		Private Frequency_pint As UInt32 = MINIMUM_FREQUENCY
	#tag EndProperty

	#tag Property, Flags = &h21
		Private ValidFrequencies_pint() As UInt32
	#tag EndProperty


	#tag Constant, Name = CRYSTAL_FREQUENCY, Type = Double, Dynamic = False, Default = \"32768", Scope = Private
	#tag EndConstant

	#tag Constant, Name = FREQUENCY_STEP, Type = Double, Dynamic = False, Default = \"200000", Scope = Private
	#tag EndConstant

	#tag Constant, Name = INTERMEDIATE_FREQUENCY, Type = Double, Dynamic = False, Default = \"225000", Scope = Private
	#tag EndConstant

	#tag Constant, Name = MAXIMUM_FREQUENCY, Type = Double, Dynamic = False, Default = \"107900000", Scope = Private
	#tag EndConstant

	#tag Constant, Name = MINIMUM_FREQUENCY, Type = Double, Dynamic = False, Default = \"87500000", Scope = Private
	#tag EndConstant


	#tag ViewBehavior
		#tag ViewProperty
			Name="Name"
			Visible=true
			Group="ID"
			InitialValue=""
			Type="String"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="Index"
			Visible=true
			Group="ID"
			InitialValue="-2147483648"
			Type="Integer"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="Super"
			Visible=true
			Group="ID"
			InitialValue=""
			Type="String"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="Left"
			Visible=true
			Group="Position"
			InitialValue="0"
			Type="Integer"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="Top"
			Visible=true
			Group="Position"
			InitialValue="0"
			Type="Integer"
			EditorType=""
		#tag EndViewProperty
	#tag EndViewBehavior
End Class
#tag EndClass
