#tag Module
Protected Module MISC_INTERNAL
	#tag Method, Flags = &h0
		Function BeyondTolerance(Number1_dbl As Double, Number2_dbl As Double, Tolerance_dbl As Double) As Boolean
		  
		  ' Determines if the difference between two floating point numbers (doubles) are
		  ' beyond a specified tolerance.
		  '
		  If Abs(Number1_dbl - Number2_dbl) >= Tolerance_dbl Then
		    Return True
		  Else
		    Return False
		  End If
		  
		End Function
	#tag EndMethod

	#tag Method, Flags = &h0
		Sub Chirp(ChirpCount_int As Integer = 1)
		  
		  ' Chirp the piezo.
		  '
		  For Index_int As Integer = 1 To ChirpCount_int
		    GPIO.DigitalWrite(ConfigPinBeeper_gint, GPIO.ON)
		    GPIO.Delay(25)
		    GPIO.DigitalWrite(ConfigPinBeeper_gint, GPIO.OFF)
		    GPIO.Delay(25)
		  Next Index_int
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Sub ConfigLoad()
		  
		  Var INI_obj As INI
		  
		  ' Create INI object
		  '
		  INI_obj = New INI(INIFile_gfi)
		  
		  ' Load configuration
		  '
		  INI_obj.Load
		  
		  ' Colors
		  '
		  ConfigWindowBackground_gclr = INI_obj.Get("COLORS", "window_background" , &c10101000)
		  ConfigDialogBackground_gclr = INI_obj.Get("COLORS", "dialog_background" , &c10101000)
		  ConfigButtonCaption_gclr    = INI_obj.Get("COLORS", "button_caption"    , &cFFFFFF00)
		  ConfigButtonFaceUp_gclr     = INI_obj.Get("COLORS", "button_face_up"    , &c00000000)
		  ConfigButtonFaceDown_gclr   = INI_obj.Get("COLORS", "button_face_down"  , &c00000000)
		  ConfigButtonBorderUp_gclr   = INI_obj.Get("COLORS", "button_border_up"  , &cA0A0A000)
		  ConfigButtonBorderDown_gclr = INI_obj.Get("COLORS", "button_border_down", &cFF000000)
		  ConfigDashboardRed_gclr     = INI_obj.Get("COLORS", "dashboard_red"     , &cFF000000)
		  ConfigDashboardGreen_gclr   = INI_obj.Get("COLORS", "dashboard_green"   , &c00FF0000)
		  ConfigDashboardYellow_gclr  = INI_obj.Get("COLORS", "dashboard_yellow"  , &cFFFF0000)
		  ConfigDashboardShadow_gclr  = INI_obj.Get("COLORS", "dashboard_shadow"  , &c00000000)
		  
		  ' FM Radio 
		  '
		  ConfigLastTuned_gstr       = INI_obj.Get("RADIO", "last_tuned"       , ""      )
		  ConfigRadioPreset1_gstr    = INI_obj.Get("RADIO", "preset_1"         , ""      )
		  ConfigRadioPreset2_gstr    = INI_obj.Get("RADIO", "preset_2"         , ""      )
		  ConfigRadioPreset3_gstr    = INI_obj.Get("RADIO", "preset_3"         , ""      )
		  ConfigRadioPreset4_gstr    = INI_obj.Get("RADIO", "preset_4"         , ""      )
		  ConfigSearchStopLevel_gstr = INI_obj.Get("RADIO", "search_stop_level", "medium")
		  
		  ' Music Player
		  '
		  ConfigMusicDefaultQueryFile_gstr = INI_obj.Get("MUSIC", "default_query_file", "(Everything)")
		  
		  ' Movie Player
		  '
		  ConfigMovieDefaultQueryFile_gstr = INI_obj.Get("MOVIE", "default_query_file", "(Everything)")
		  ConfigMovieAspectMode_gstr       = INI_obj.Get("MOVIE", "aspect_mode"       , "fill"        )
		  
		  ' GPS Recorder
		  '
		  ConfigGPSDevice_gstr         = INI_obj.Get("GPS", "device"              , ""    )
		  ConfigGPSLogs_gstr           = INI_obj.Get("GPS", "logs"                , ""    )
		  ConfigGPSMinLatMovement_gdbl = INI_obj.Get("GPS", "log_min_lat_movement", 0.0001)
		  ConfigGPSMinLonMovement_gdbl = INI_obj.Get("GPS", "log_min_lon_movement", 0.0001)
		  ConfigGPSLogFrequency_gint   = INI_obj.Get("GPS", "log_frequency"       , 5     )
		  
		  ' Diagnostics
		  '
		  ConfigOBDDevice_gstr = INI_obj.Get("OBD", "device", "")
		  
		  ' GPIO
		  '
		  ConfigPinAmplifier_gint       = INI_obj.Get("GPIO", "pin_amplifier"       , 17)
		  ConfigPinAntenna_gint         = INI_obj.Get("GPIO", "pin_antenna"         , 18)
		  ConfigPinAudioSelector_gint   = INI_obj.Get("GPIO", "pin_audio_selector"  , 12)
		  ConfigPinBeeper_gint          = INI_obj.Get("GPIO", "pin_beeper"          , 26)
		  ConfigPinShutdownRequest_gint = INI_obj.Get("GPIO", "pin_shutdown_request", 24)
		  
		  ' GUI
		  '
		  ConfigMouseCursor_gstr  = INI_obj.Get("GUI", "mouse_cursor" , "hide"    )
		  ConfigLastFunction_gstr = INI_obj.Get("GUI", "last_function", "fm_radio")
		  
		  ' Destroy INI object
		  '
		  INI_obj = Nil
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Sub ConfigSave()
		  
		  Var INI_obj As INI
		  
		  ' Create INI object
		  '
		  INI_obj = New INI(INIFile_gfi)
		  
		  ' Colors
		  '
		  INI_obj.Put("COLORS", "window_background" , ConfigWindowBackground_gclr)
		  INI_obj.Put("COLORS", "dialog_background" , ConfigDialogBackground_gclr)
		  INI_obj.Put("COLORS", "button_caption"    , ConfigButtonCaption_gclr   )
		  INI_obj.Put("COLORS", "button_face_up"    , ConfigButtonFaceUp_gclr    )
		  INI_obj.Put("COLORS", "button_face_down"  , ConfigButtonFaceDown_gclr  )
		  INI_obj.Put("COLORS", "button_border_up"  , ConfigButtonBorderUp_gclr  )
		  INI_obj.Put("COLORS", "button_border_down", ConfigButtonBorderDown_gclr)
		  INI_obj.Put("COLORS", "dashboard_red"     , ConfigDashboardRed_gclr    )
		  INI_obj.Put("COLORS", "dashboard_green"   , ConfigDashboardGreen_gclr  )
		  INI_obj.Put("COLORS", "dashboard_yellow"  , ConfigDashboardYellow_gclr )
		  INI_obj.Put("COLORS", "dashboard_shadow"  , ConfigDashboardShadow_gclr )
		  
		  ' FM Radio 
		  '
		  INI_obj.Put("RADIO", "last_tuned"       , ConfigLastTuned_gstr      )
		  INI_obj.Put("RADIO", "preset_1"         , ConfigRadioPreset1_gstr   )
		  INI_obj.Put("RADIO", "preset_2"         , ConfigRadioPreset2_gstr   )
		  INI_obj.Put("RADIO", "preset_3"         , ConfigRadioPreset3_gstr   )
		  INI_obj.Put("RADIO", "preset_4"         , ConfigRadioPreset4_gstr   )
		  INI_obj.Put("RADIO", "search_stop_level", ConfigSearchStopLevel_gstr)
		  
		  ' Music Player
		  '
		  INI_obj.Put("MUSIC", "default_query_file", ConfigMusicDefaultQueryFile_gstr)
		  
		  ' Movie Player
		  '
		  INI_obj.Put("MOVIE", "default_query_file", ConfigMovieDefaultQueryFile_gstr)
		  INI_obj.Put("MOVIE", "aspect_mode"       , ConfigMovieAspectMode_gstr      )
		  
		  ' GPS Recorder
		  '
		  INI_obj.Put("GPS", "device"              , ConfigGPSDevice_gstr        )
		  INI_obj.Put("GPS", "logs"                , ConfigGPSLogs_gstr          )
		  INI_obj.Put("GPS", "log_min_lat_movement", ConfigGPSMinLatMovement_gdbl)
		  INI_obj.Put("GPS", "log_min_lon_movement", ConfigGPSMinLonMovement_gdbl)
		  INI_obj.Put("GPS", "log_frequency"       , ConfigGPSLogFrequency_gint  )
		  
		  ' Diagnostics
		  '
		  INI_obj.Put("OBD", "device", ConfigOBDDevice_gstr)
		  
		  ' GPIO
		  '
		  INI_obj.Put("GPIO", "pin_amplifier"       , ConfigPinAmplifier_gint      )
		  INI_obj.Put("GPIO", "pin_antenna"         , ConfigPinAntenna_gint        )
		  INI_obj.Put("GPIO", "pin_audio_selector"  , ConfigPinAudioSelector_gint  )
		  INI_obj.Put("GPIO", "pin_beeper"          , ConfigPinBeeper_gint         )
		  INI_obj.Put("GPIO", "pin_shutdown_request", ConfigPinShutdownRequest_gint)
		  
		  ' GUI
		  '
		  INI_obj.Put("GUI", "mouse_cursor" , ConfigMouseCursor_gstr )
		  INI_obj.Put("GUI", "last_function", ConfigLastFunction_gstr)
		  
		  ' Save configuration
		  '
		  INI_obj.Save
		  
		  ' Destroy INI object
		  '
		  INI_obj = Nil
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Sub DeleteFile(File_str As String)
		  
		  Var Temp_fi As FolderItem = New FolderItem(File_str, FolderItem.PathModes.Native)
		  
		  Temp_fi.Remove
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Sub DrawDecorativeFrame(g As Graphics, Size_str As String)
		  
		  Select Case Size_str
		    
		  Case "small"
		    
		    ' Yellow-out the entire display area.
		    '
		    g.DrawingColor = ConfigDashboardYellow_gclr
		    g.FillRoundRectangle(0, 0, 510, 186, 36, 36)
		    
		    ' Black-out the inner display area.
		    '
		    g.DrawingColor = &c000000
		    g.FillRoundRectangle(5, 5, 500, 176, 31, 31)
		    
		  Case "large"
		    
		    ' Yellow-out the entire display area.
		    '
		    g.DrawingColor = ConfigDashboardYellow_gclr
		    g.FillRoundRectangle(0, 0, 510, 273, 36, 36)
		    
		    ' Black-out the inner display area.
		    '
		    g.DrawingColor = &c000000
		    g.FillRoundRectangle(5, 5, 500, 263, 31, 31)
		    
		  End Select
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Function FileExists(MethodName_str As String, File_str As String) As Boolean
		  
		  Var Temp_fi As FolderItem = New FolderItem(File_str, FolderItem.PathModes.Native)
		  If (Temp_fi <> Nil) And (Temp_fi.Exists = True) And (Temp_fi.IsFolder = False) Then
		    Return True
		  Else
		    Return False
		  End If
		  
		  
		  Exception Error As RunTimeException
		    Var AckBox_obj As ack_box = New ack_box
		    AckBox_obj.ShowModal("Error in: " + MethodName_str, "Error Number: " + Error.ErrorNumber.ToText + EndOfLine + "Error Message: " + LinuxSysErrorLookup(Error.ErrorNumber) + EndOfLine + EndOfLine + "Application will close.", ack_box.MsgTypes.Emergency)
		    Quit
		    
		End Function
	#tag EndMethod

	#tag Method, Flags = &h0
		Function FileToString(MethodName_str As String, Filename_str As String) As String
		  
		  ' Read data from file into string.
		  '
		  Var Temp_fi As FolderItem = New FolderItem(Filename_str, FolderItem.PathModes.Native)
		  Var Temp_tis As TextInputStream = TextInputStream.Open(Temp_fi)
		  
		  Var Data_str As String = Temp_tis.ReadAll
		  Temp_tis.Close
		  
		  Return Data_str
		  
		  
		  Exception Error As RunTimeException
		    Var AckBox_obj As ack_box = New ack_box
		    AckBox_obj.ShowModal("Error in: " + MethodName_str, "Error Number: " + Error.ErrorNumber.ToText + EndOfLine + "Error Message: " + LinuxSysErrorLookup(Error.ErrorNumber) + EndOfLine + EndOfLine + "Application will close.", ack_box.MsgTypes.Emergency)
		    Quit
		    
		End Function
	#tag EndMethod

	#tag Method, Flags = &h0
		Function FormatMicroSeconds(uS_int As UInt64) As String
		  
		  Var H_str, M_str, S_str As String
		  
		  
		  ' Convert MicroSeconds (uS) into Seconds.
		  '
		  uS_int = uS_int / 1000000
		  
		  ' Break seconds into hours, minutes, seconds.
		  '
		  H_str = "0" + Str(uS_int \ 3600)
		  H_str = H_str.Right(2)
		  
		  uS_int = uS_int Mod 3600
		  
		  M_str = "0" + Str(uS_int \ 60)
		  M_str = M_str.Right(2)
		  
		  uS_int = uS_int Mod 60
		  
		  S_str = "0" + Str(uS_int)
		  S_str = S_str.Right(2)
		  
		  Return H_str + ":" + M_str + ":" + S_str
		  
		End Function
	#tag EndMethod

	#tag Method, Flags = &h0
		Function GetVideoDisplayZone(X_int As Integer, Y_int As Integer, W_int As Integer, H_int As Integer) As Integer
		  
		  Var Row_int As Integer
		  Var Col_int As Integer
		  
		  
		  ' The video display is treated as if it contains nine
		  ' touch zones:
		  '
		  '     +-----------------------+
		  '     |   0   |   1   |   2   |
		  '     |-------+-------+-------|
		  '     |   3   |   4   |   5   |
		  '     |-------+-------+-------|
		  '     |   6   |   7   |   8   |
		  '     +-----------------------+
		  
		  ' This method returns a zone number depending upon the 
		  ' point defined by X and Y within the area defined by
		  ' W and H.
		  
		  
		  ' Is point outside of the area?
		  '
		  If ((X_int < 0) Or (X_int > W_int)) Or ((Y_int < 0) Or (Y_int > H_int)) Then
		    Return -1
		    Exit Function
		  End If
		  
		  ' Get row.
		  '
		  If (X_int >= (W_int * 0.00)) And (X_int <  (W_int * 0.33)) Then
		    Col_int = 0
		  End If
		  
		  If (X_int >= (W_int * 0.33)) And (X_int <  (W_int * 0.66)) Then
		    Col_int = 1
		  End If
		  
		  If (X_int >= (W_int * 0.66)) And (X_int <= (W_int * 1.00)) Then
		    Col_int = 2
		  End If
		  
		  ' Get column.
		  '
		  If (Y_int >= (H_int * 0.00)) And (Y_int <  (H_int * 0.33)) Then
		    Row_int = 0
		  End If
		  
		  If (Y_int >= (H_int * 0.33)) And (Y_int <  (H_int * 0.66)) Then
		    Row_int = 1
		  End If
		  
		  If (Y_int >= (H_int * 0.66)) And (Y_int <= (H_int * 1.00)) Then
		    Row_int = 2
		  End If
		  
		  ' Return zone number.
		  '
		  Return (Row_int * 3) + Col_int
		  
		End Function
	#tag EndMethod

	#tag Method, Flags = &h0
		Function ISO8601Timestamp() As String
		  
		  Var GMT_tz As New TimeZone("GMT")
		  Var NOW_dt As DateTime
		  Var S As String
		  
		  
		  ' GPX track points require a GMT based timestamp in the ISO 8601
		  ' format: YYYY-MM-DDTHH:MM:SSZ
		  '
		  NOW_dt = DateTime.Now(GMT_tz)
		  
		  S = NOW_dt.Year.ToString _
		  + "-" _
		  + NOW_dt.Month.ToString.PadLeftWith(2, "0") _
		  + "-" _
		  + NOW_dt.Day.ToString.PadLeftWith(2, "0") _
		  + "T" _
		  + NOW_dt.Hour.ToString.PadLeftWith(2, "0") _
		  + ":" _
		  + NOW_dt.Minute.ToString.PadLeftWith(2, "0") _
		  + ":" _
		  + NOW_dt.Second.ToString.PadLeftWith(2, "0") _
		  + "Z"
		  
		  Return S
		  
		End Function
	#tag EndMethod

	#tag Method, Flags = &h0
		Sub ReColor(pic As Picture, Search_clr As Color, Replace_clr As Color)
		  
		  Var RGBSurface_obj As RGBSurface = pic.RGBSurface
		  Var LastX_int As Integer = pic.Width - 1
		  Var LastY_int As Integer = pic.Height - 1
		  
		  
		  ' This routine scans a picture for pixels of a specified color and 
		  ' recolors those pixels with another specified color. This is useful
		  ' for altering the various pictures used for things like 7 segment
		  ' digital displays, progress bars, signal strength indicator, etc.
		  '
		  For Y_int As Integer = 0 To LastY_int
		    For X_int As Integer = 0 To LastX_int
		      If RGBSurface_obj.Pixel(X_int,Y_int) = Search_clr  Then
		        RGBSurface_obj.Pixel(X_int,Y_int) = Replace_clr
		      End if
		    Next X_int
		  Next Y_int
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Sub StringToFile(MethodName_str As String, Filename_str As String, Data_str As String)
		  
		  ' Write data from string into file.
		  '
		  Var Temp_fi As FolderItem = New FolderItem(Filename_str, FolderItem.PathModes.Native)
		  Var Temp_tos As TextOutputStream = TextOutputStream.Create(Temp_fi)
		  
		  Temp_tos.Write(Data_str)
		  Temp_tos.Close
		  
		  
		  Exception Error As RunTimeException
		    Var AckBox_obj As ack_box = New ack_box
		    AckBox_obj.ShowModal("Error in: " + MethodName_str, "Error Number: " + Error.ErrorNumber.ToText + EndOfLine + "Error Message: " + LinuxSysErrorLookup(Error.ErrorNumber) + EndOfLine + EndOfLine + "Application will close.", ack_box.MsgTypes.Emergency)
		    Quit
		    
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Function Timestamp() As String
		  
		  Var NOW_dt As DateTime
		  Var S As String
		  
		  
		  ' Create a date and time string suitable for use as a filename.
		  ' Example: YYYY-MM-DD-HH-MM-SS
		  '
		  NOW_dt = DateTime.Now
		  
		  S = NOW_dt.Year.ToString _
		  + NOW_dt.Month.ToString.PadLeftWith(2, "0") _
		  + NOW_dt.Day.ToString.PadLeftWith(2, "0") _
		  + NOW_dt.Hour.ToString.PadLeftWith(2, "0") _
		  + NOW_dt.Minute.ToString.PadLeftWith(2, "0") _
		  + NOW_dt.Second.ToString.PadLeftWith(2, "0")
		  
		  Return S
		  
		End Function
	#tag EndMethod


	#tag Property, Flags = &h0
		AmplifierMgr_gobj As amplifier_mgr = Nil
	#tag EndProperty

	#tag Property, Flags = &h0
		AntennaMgr_gobj As antenna_mgr = Nil
	#tag EndProperty

	#tag Property, Flags = &h0
		AudioSwitchMgr_gobj As audio_switch_mgr = Nil
	#tag EndProperty

	#tag Property, Flags = &h0
		ConfigButtonBorderDown_gclr As Color = &cFF0000
	#tag EndProperty

	#tag Property, Flags = &h0
		ConfigButtonBorderUp_gclr As Color = &cA0A0A0
	#tag EndProperty

	#tag Property, Flags = &h0
		ConfigButtonCaption_gclr As Color = &cFFFFFF
	#tag EndProperty

	#tag Property, Flags = &h0
		ConfigButtonFaceDown_gclr As Color = &c000000
	#tag EndProperty

	#tag Property, Flags = &h0
		ConfigButtonFaceUp_gclr As Color = &c000000
	#tag EndProperty

	#tag Property, Flags = &h0
		ConfigDashboardGreen_gclr As Color
	#tag EndProperty

	#tag Property, Flags = &h0
		ConfigDashboardRed_gclr As Color
	#tag EndProperty

	#tag Property, Flags = &h0
		ConfigDashboardShadow_gclr As Color
	#tag EndProperty

	#tag Property, Flags = &h0
		ConfigDashboardYellow_gclr As Color
	#tag EndProperty

	#tag Property, Flags = &h0
		ConfigDialogBackground_gclr As Color
	#tag EndProperty

	#tag Property, Flags = &h0
		ConfigGPSDevice_gstr As String
	#tag EndProperty

	#tag Property, Flags = &h0
		ConfigGPSLogFrequency_gint As Integer
	#tag EndProperty

	#tag Property, Flags = &h0
		ConfigGPSLogs_gstr As String
	#tag EndProperty

	#tag Property, Flags = &h0
		ConfigGPSMinLatMovement_gdbl As Double
	#tag EndProperty

	#tag Property, Flags = &h0
		ConfigGPSMinLonMovement_gdbl As Double
	#tag EndProperty

	#tag Property, Flags = &h0
		ConfigLastFunction_gstr As String
	#tag EndProperty

	#tag Property, Flags = &h0
		ConfigLastTuned_gstr As String
	#tag EndProperty

	#tag Property, Flags = &h0
		ConfigMouseCursor_gstr As String
	#tag EndProperty

	#tag Property, Flags = &h0
		ConfigMovieAspectMode_gstr As String
	#tag EndProperty

	#tag Property, Flags = &h0
		ConfigMovieDefaultQueryFile_gstr As String
	#tag EndProperty

	#tag Property, Flags = &h0
		ConfigMusicDefaultQueryFile_gstr As String
	#tag EndProperty

	#tag Property, Flags = &h0
		ConfigOBDDevice_gstr As String
	#tag EndProperty

	#tag Property, Flags = &h0
		ConfigPinAmplifier_gint As Integer
	#tag EndProperty

	#tag Property, Flags = &h0
		ConfigPinAntenna_gint As Integer
	#tag EndProperty

	#tag Property, Flags = &h0
		ConfigPinAudioSelector_gint As Integer
	#tag EndProperty

	#tag Property, Flags = &h0
		ConfigPinBeeper_gint As Integer
	#tag EndProperty

	#tag Property, Flags = &h0
		ConfigPinShutdownRequest_gint As Integer
	#tag EndProperty

	#tag Property, Flags = &h0
		ConfigRadioPreset1_gstr As String
	#tag EndProperty

	#tag Property, Flags = &h0
		ConfigRadioPreset2_gstr As String
	#tag EndProperty

	#tag Property, Flags = &h0
		ConfigRadioPreset3_gstr As String
	#tag EndProperty

	#tag Property, Flags = &h0
		ConfigRadioPreset4_gstr As String
	#tag EndProperty

	#tag Property, Flags = &h0
		ConfigSearchStopLevel_gstr As String
	#tag EndProperty

	#tag Property, Flags = &h0
		ConfigWindowBackground_gclr As Color
	#tag EndProperty

	#tag Property, Flags = &h0
		MainWindow_gobj As main_window = Nil
	#tag EndProperty


	#tag ViewBehavior
		#tag ViewProperty
			Name="ConfigDashboardShadow_gclr"
			Visible=false
			Group="Behavior"
			InitialValue="&c000000"
			Type="Color"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="ConfigDashboardGreen_gclr"
			Visible=false
			Group="Behavior"
			InitialValue="&c000000"
			Type="Color"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="ConfigDashboardRed_gclr"
			Visible=false
			Group="Behavior"
			InitialValue="&c000000"
			Type="Color"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="ConfigDashboardYellow_gclr"
			Visible=false
			Group="Behavior"
			InitialValue="&c000000"
			Type="Color"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="ConfigGPSDevice_gstr"
			Visible=false
			Group="Behavior"
			InitialValue=""
			Type="String"
			EditorType="MultiLineEditor"
		#tag EndViewProperty
		#tag ViewProperty
			Name="ConfigGPSLogs_gstr"
			Visible=false
			Group="Behavior"
			InitialValue=""
			Type="String"
			EditorType="MultiLineEditor"
		#tag EndViewProperty
		#tag ViewProperty
			Name="ConfigMovieAspectMode_gstr"
			Visible=false
			Group="Behavior"
			InitialValue=""
			Type="String"
			EditorType="MultiLineEditor"
		#tag EndViewProperty
		#tag ViewProperty
			Name="ConfigMovieDefaultQueryFile_gstr"
			Visible=false
			Group="Behavior"
			InitialValue=""
			Type="String"
			EditorType="MultiLineEditor"
		#tag EndViewProperty
		#tag ViewProperty
			Name="ConfigMusicDefaultQueryFile_gstr"
			Visible=false
			Group="Behavior"
			InitialValue=""
			Type="String"
			EditorType="MultiLineEditor"
		#tag EndViewProperty
		#tag ViewProperty
			Name="ConfigOBDDevice_gstr"
			Visible=false
			Group="Behavior"
			InitialValue=""
			Type="String"
			EditorType="MultiLineEditor"
		#tag EndViewProperty
		#tag ViewProperty
			Name="ConfigPinAmplifier_gint"
			Visible=false
			Group="Behavior"
			InitialValue=""
			Type="Integer"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="ConfigPinAntenna_gint"
			Visible=false
			Group="Behavior"
			InitialValue=""
			Type="Integer"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="ConfigPinAudioSelector_gint"
			Visible=false
			Group="Behavior"
			InitialValue=""
			Type="Integer"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="ConfigPinBeeper_gint"
			Visible=false
			Group="Behavior"
			InitialValue=""
			Type="Integer"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="ConfigRadioPreset1_gstr"
			Visible=false
			Group="Behavior"
			InitialValue=""
			Type="String"
			EditorType="MultiLineEditor"
		#tag EndViewProperty
		#tag ViewProperty
			Name="ConfigRadioPreset2_gstr"
			Visible=false
			Group="Behavior"
			InitialValue=""
			Type="String"
			EditorType="MultiLineEditor"
		#tag EndViewProperty
		#tag ViewProperty
			Name="ConfigRadioPreset3_gstr"
			Visible=false
			Group="Behavior"
			InitialValue=""
			Type="String"
			EditorType="MultiLineEditor"
		#tag EndViewProperty
		#tag ViewProperty
			Name="ConfigRadioPreset4_gstr"
			Visible=false
			Group="Behavior"
			InitialValue=""
			Type="String"
			EditorType="MultiLineEditor"
		#tag EndViewProperty
		#tag ViewProperty
			Name="Index"
			Visible=true
			Group="ID"
			InitialValue="-2147483648"
			Type="Integer"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="Left"
			Visible=true
			Group="Position"
			InitialValue="0"
			Type="Integer"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="Name"
			Visible=true
			Group="ID"
			InitialValue=""
			Type="String"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="Super"
			Visible=true
			Group="ID"
			InitialValue=""
			Type="String"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="Top"
			Visible=true
			Group="Position"
			InitialValue="0"
			Type="Integer"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="ConfigButtonBorderDown_gclr"
			Visible=false
			Group="Behavior"
			InitialValue="&cFF0000"
			Type="Color"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="ConfigButtonBorderUp_gclr"
			Visible=false
			Group="Behavior"
			InitialValue="&cA0A0A0"
			Type="Color"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="ConfigButtonCaption_gclr"
			Visible=false
			Group="Behavior"
			InitialValue="&cFFFFFF"
			Type="Color"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="ConfigButtonFaceDown_gclr"
			Visible=false
			Group="Behavior"
			InitialValue="&c000000"
			Type="Color"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="ConfigButtonFaceUp_gclr"
			Visible=false
			Group="Behavior"
			InitialValue="&c000000"
			Type="Color"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="ConfigWindowBackground_gclr"
			Visible=false
			Group="Behavior"
			InitialValue="&c30303000"
			Type="Color"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="ConfigDialogBackground_gclr"
			Visible=false
			Group="Behavior"
			InitialValue="&c000000"
			Type="Color"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="ConfigLastTuned_gstr"
			Visible=false
			Group="Behavior"
			InitialValue=""
			Type="String"
			EditorType="MultiLineEditor"
		#tag EndViewProperty
		#tag ViewProperty
			Name="ConfigMouseCursor_gstr"
			Visible=false
			Group="Behavior"
			InitialValue=""
			Type="String"
			EditorType="MultiLineEditor"
		#tag EndViewProperty
		#tag ViewProperty
			Name="ConfigSearchStopLevel_gstr"
			Visible=false
			Group="Behavior"
			InitialValue=""
			Type="String"
			EditorType="MultiLineEditor"
		#tag EndViewProperty
		#tag ViewProperty
			Name="ConfigGPSMinLatMovement_gdbl"
			Visible=false
			Group="Behavior"
			InitialValue=""
			Type="Double"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="ConfigGPSMinLonMovement_gdbl"
			Visible=false
			Group="Behavior"
			InitialValue=""
			Type="Double"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="ConfigPinShutdownRequest_gint"
			Visible=false
			Group="Behavior"
			InitialValue=""
			Type="Integer"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="ConfigGPSLogFrequency_gint"
			Visible=false
			Group="Behavior"
			InitialValue=""
			Type="Integer"
			EditorType=""
		#tag EndViewProperty
		#tag ViewProperty
			Name="ConfigLastFunction_gstr"
			Visible=false
			Group="Behavior"
			InitialValue=""
			Type="String"
			EditorType="MultiLineEditor"
		#tag EndViewProperty
	#tag EndViewBehavior
End Module
#tag EndModule
